/*
  nitf.h - definitions for National Imagery Transmission Format files

*/

/* see MIL-STD-2500A p 19 and 22 */
struct nitf_header
{
  char fhdr[9];
  char clevel[2];
  char stype[4];
  char ostaid[10];
  char fdt[14];
  char ftitle[80];
  char fsclas[1];
  char fscode[40];
  char fsctlh[40];
  char fsrel[40];
  char fscaut[20];
  char fsctln[20];
  char fsdwng[6];
  char fsdevt[40];
  int fscop;
  int fscpys;
  char encryp[1];
  char oname[27];
  char ophone[18];
  int fl;
  int hl;
  int numi;
  struct image_link *ilink;	/* array of links to images */
  int nums;
  struct symbol_link *slink;	/* array of links to symbols */
  int numl;
  struct label_link *llink;	/* array of links to labels */
  int numt;
  struct text_link *tlink;	/* array of links to text items */
  int numd;
  struct des_link *dlink;	/* array of links to data extension
				   segments */
  int numr;
  struct res_link *rlink;	/* array of links to reserved
				   extension segments */
  int udhdl;
  int udhofl;
  void *udhd;
  int xhdl;
  void *xhd;
};

struct image_link
{
    int lish;			/* length of image subheader */
    int li;			/* length of image */
    struct image_subheader *ish;
    char *im;
};

/* see MIL-STD-2500A p 31 and 35 */
struct image_subheader
{
  char im[2];
  char iid[10];
  char idatim[14];
  char tgtid[17];
  char ititle[80];
  char isclas[1];
  char iscode[40];
  char isctlh[40];
  char isrel[40];
  char iscaut[20];
  char isctln[20];
  char isdwng[6];
  char isdevt[40];
  char encryp[1];
  char isorce[42];
  int nrows;
  int ncols;
  char pvtype[3];
  char irep[8];
  char icat[8];
  int abpp;
  char pjust[1];
  char icords[1];
  char igeolo[60];
  int nicom;
  char *icom;
  char ic[2];
  char comrat[4];
  int nbands;
  struct band_st *band;
  int isync;
  char imode[1];
  int nbpr;
  int nbpc;
  int nppbh;
  int nppbv;
  int nbpp;
  int idlvl;
  int ialvl;
  int ilocr;
  int ilocc;
  char imag[4];
  int udidl;
  int udofl;
  void *udid;
  int ixshdl;
  int ixsofl;
  void *ixshd;  
};

struct band_st
{
  char irepband[2];
  char isubcat[6];
  char ifc[1];
  char imflt[3];
  int nluts;
  int nelut;
  char *lutd;
};

struct symbol_link
{
    int lssh;			/* length of symbol subheader */
    int ls;			/* length of symbol */
    struct symbol_subheader *sh;
    struct symbol *sym;
};

/* see MIL-STD-2500A p 61 and 62 */
struct symbol_subheader
{
};

struct label_link
{
    int llsh;			/* length of label subheader */
    int ll;			/* length of label */
    struct label_subheader *sh;
    struct label *lab;
};

/* see MIL-STD-2500A p 71 and 72*/
struct label_subheader
{
};

struct text_link
{
    int ltsh;			/* length of text subheader */
    int lt;			/* length of text */
    struct text_subheader *sh;
    struct text *tm;
};

/* see MIL-STD-2500A p 76 and 77*/
struct text_subheader
{
};

struct des_link
{
    int ldsh;			/* length of des subheader */
    int ld;			/* length of des */
    struct des_subheader *sh;
    struct des *dm;
};

/* see MIL-STD-2500A p 83 */
struct des_subheader
{
  char de[2];
  char destag[25];
  char desver[2];
  char desclas[1];
  char descode[40];
  char desctlh[40];
  char desrel[40];
  char descaut[20];
  char destcln[20];
  char desdwng[6];
  char desdevt[40];
  char desoflw[6];
  int desitem;
  int desshl;
  void *desshf;
  void *desdata;
};

struct res_link
{
    int lrsh;			/* length of res subheader */
    int lr;			/* length of res */
    struct res_subheader *sh;
    struct res *rm;
};

/* see MIL-STD-2500A p 81 */
struct res_subheader
{
  char retag[6];
  int rel;
  void *redata;
};


enum {NITF_OK, NITF_FAIL, NITF_BADFILE};

int readheader(FILE *ifile, struct nitf_header **hdrp);
int writeppm(FILE *ofile, struct nitf_header *hdr, struct image_link *ilink);

void *xrealloc(void *ptr, unsigned n);
void *xmalloc(unsigned n);
void fail(char *s);
